/* This tool is used to convert paths containing extensible patterns */
/* to absolute paths, as the library function realpath(3) does.      */
/* (c)2002 SerNet GmbH, (c)2003 Oskar Schirmer                       */
/* licensed under the term of the GNU Public License (GPL)           */
/*   see file COPYING for details                                    */

#include <stdio.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <string.h>

const static char *version = "1.2";

static void do_rp (const char *p)
{
	int i;
	char *r;
	char t[PATH_MAX];
	r = realpath (p,t);
	if (r != NULL) {
		puts (r);
	} else {
		i = errno;
		perror (p);
		exit (i);
	}
}

static void do_stdin (void)
{
	char s[PATH_MAX];
	while (! feof (stdin)) {
		if (fgets (s, sizeof (s), stdin) != NULL) {
			if ((strlen (s) > 0) && (s[strlen(s)-1] == '\n')) {
				s[strlen(s)-1] = 0;
			}
			do_rp (s);
		} 
	}
}

static void do_help (char *av0)
{
	fprintf (stderr, "usage: %s [OPTION/PATH]...\n"
"Convert the paths arguments to absolute paths, print them on stdout.\n"
"  --\t\tall the following arguments are non-options\n"
"  -\t\tread paths from stdin one per line and print the corresponding\n"
"\t\t\tabsolute paths to stdout\n"
"  --help\tdisplay this help and exit\n"
"  --version\tdisplay version and exit\n",
		av0);
}

int main (int argc, char *argv[])
{
	int optionable = 1;
	int stdindone = 0;
	char **av = argv;
	while (--argc > 0) {
		av += 1;
		if (optionable && (av[0][0] == '-')) {
			if (av[0][1] == 0) {
				if (stdindone) {
					fprintf (stderr,
				"%s failed: cannot process stdin twice.\n",
						argv[0]);
					return 1;
				} else {
					do_stdin ();
					stdindone = 1;
				}
			} else if (!strcmp (&(av[0][1]), "-")) {
				optionable = 0;
			} else if (!strcmp (&(av[0][1]), "-help")) {
				do_help (argv[0]);
				return 0;
			} else if (!strcmp (&(av[0][1]), "-version")) {
				fprintf (stderr, "v%s\n", version);
				return 0;
			} else {
				fprintf (stderr,
					"%s failed: unknown option %s\n",
					argv[0], av[0]);
				do_help (argv[0]);
				return 1;
			}
		} else {
			do_rp (av[0]);
		}
	}
	return 0;
}


